nest_sim <- function(isim,C, phi_int, phi_cattle, phi_stage, phi_robel,
                     p_int, p_cattle, p_stage, p_robel,
                     pi_cattle, m_rob, s_rob, m_a1, s_a1) {

 #  change ncpu to number of cores*threads/core
 #   you might want to change ncpu to 1 or 2 if you have a laptop
 #   with only 2 cores and no hyperthreading.

  if (!exists('zGH')) nestAbund.default()
  set.seed(12347)   ;  GHnodes<<-15; Q<<-GHnodes

   Visits=seq(1,100,5)   # Timing of visits (starts with 1)  10 visits
   Last=max(Visits); T=length(Visits); NT<<-T; DaySampl<<-matrix(Visits,nrow=1,ncol=T)

   fname=paste('nest_sim',isim,'.csv',sep='')

   nestID=Site=Max.Clutch.Size=AgeFound=Fate=
     Cattle=Robel=AgeDay1=FirstFound=IsFound=LastPresent=Ent=LP=NULL

   hst<<-matrix(0,C,100)
   nestID=1:C; Site=rep(1,C); kclutch=10; t=1:34
   Max.Clutch.Size=rep(kclutch,C); # fixed clutch size
   for (k in 1:C) {
     kcattle=rbinom(1,1,pi_cattle); Cattle=c(Cattle,kcattle)
     krobel=rnorm(1,m_rob,s_rob); Robel=c(Robel,krobel)
     ka1=round(rnorm(1,m_a1,s_a1)); AgeDay1=c(AgeDay1,ka1)
     entry=(1-ka1); Ent=c(Ent,entry)
     detect = plogis(p_int + p_stage*(t<=kclutch) + p_cattle*kcattle + p_robel*krobel)
     surv = plogis(phi_int + phi_stage*(t>kclutch) + phi_cattle*kcattle + phi_robel*krobel)
     i=1;state=1;found=0;ff=aa=lp=-99;

     while (i<=34 && state==1) {
       ii=sum((entry+i)==Visits)
       if (ii>0) {
         if (found==0) { found = rbinom(1,1,detect[i]) }
         if (found==1 && ff<0) { ff=entry+i; aa=i }
         if (found==1) { lp=entry+i; hst[k,entry+i]<<-1 }
       }
       if (i<34) state = rbinom(1,1,surv[i])
       i=i+1
     }
     Fate=c(Fate,1-state); FirstFound=c(FirstFound,ff);  AgeFound=c(AgeFound,aa)
     tmp = ((ff<=Last) && (found==1)) ; IsFound=c(IsFound,tmp)
     ll=entry+i-1;  LastPresent=c(LastPresent,lp); LP=c(LP,ll)
   }

   sim = data.frame(cbind(nestID, Site,Max.Clutch.Size ,Cattle ,Robel ,
       AgeDay1,FirstFound,AgeFound,LastPresent, Fate, Ent, LP))[IsFound>0,]
   write.table(sim,fname,sep=',',row.names=F)
   xxxstd=read_input_file(fname)
   cat(mean(xdata$FirstFound),CC,'\n')
    xdata[xdata$FirstFound>xdata$LastPresent,]

   a1=nestAbund.runmod(paste('Phi()P()GAU',isim,sep=''))

return(a1)
}           #   end of nest_sim function

nestAbund.sim <- function(ncpu=1, nsims=1,  C=500,
                          phi_int=5, phi_cattle=0, phi_stage=0, phi_robel=0,
                          p_int=-1, p_cattle=0, p_stage=0, p_robel=0,
                          pi_cattle=0.5, m_rob=1.80, s_rob=1, m_a1=0, s_a1=5,
                          modl='Phi()P()GAU') {
  #' simulates data and runs nestAbund model
  #'
  #'     This function is used to simulate and analyze nest abund. data, for a given model
  #'@param ncpu (1) number of cpu cores to use
  #'@param nsims (1) number of simulations to do
  #'@param C (500) number of nests (including ones that wont be detected)
  #'@param phi_int (5) beta intercept for phi (survival)
  #'@param phi_cattle (0) beta for cattle effect on phi
  #'@param phi_stage (0) beta for stage effect on phi
  #'@param phi_robel (0) beta for robel effect on robel
  #'@param p_int (-1) beta intercept for p (detection)
  #'@param p_cattle (0) beta for cattle effect on p
  #'@param p_stage (0) beta for stage effect on p
  #'@param p_robel (0) beta for robel effect on p
  #'@param pi_cattle (0.5) proportion of nests with cattle
  #'@param m_rob (1.8) mean value of robel covariate
  #'@param s_rob (1) std.dev of robel covariate
  #'@param m_a1 (0) mean value of age at first visit
  #'@param s_a1 (5) std.dev of age at first visit
  #'@returns returns nothing.
  #'@export
  #'@details
  #'  Output for the model is saved in file.
  #'       \code{nestAbund_Phi(xxx)P(xxx)YYY.out} \code{text output from model}
  #'  where xxx denotes model chosen, YYY denotes the distribution chosen.
  #'@author J. Hines
  #'@seealso \code{\link{nestAbund.gui}}, \code{\link{nestAbund.runmod}}
  #'@examples
  #'   \dontrun{
  #'    nestAbund.sim() #  without arguments, a graphic user-interface is displayed
  #'   }
  if (ncpu>1) {
    if (sum(ls(1)=='cl')<1) {
      l=ncpu; library(snow); library(rlecuyer); cat('making cluster of',l,'\n')
      t1=system.time(cl<-makeSOCKcluster(rep("localhost",l)));
      cat('done in',t1[3],'\n')
    }
    clusterExport(cl,list=c(ls(),"ginv","rmvnorm"))
    clusterSetupRNG(cl,seed=rep(12347,6))
    t2=system.time(vv<-clusterApplyLB(cl,1:nsims,nest_sim));
    #stopCluster(cl)
  } else {
    t2=0; vv=list();
    for (isim in 1:nsims)
      t2=t2+system.time(vv[[isim]]<-nest_sim(isim,
              C, phi_int, phi_cattle, phi_stage, phi_robel,
              p_int, p_cattle, p_stage, p_robel,
              pi_cattle, m_rob, s_rob, m_a1, s_a1))
  }
  rslts1=NULL ; mnames=c('Phi()P()','Phi(stage)P()')
  for (i in 1:nsims) {
   npar=c(length(vv[[i]][[1]]$estimate),length(vv[[i]][[2]]$estimate))
   ll=c(vv[[i]][[1]]$minimum,vv[[i]][[2]]$minimum)
   aic=2*ll-2*npar; k=which.min(aic)
   dlta=aic[3-k]-aic[k]
   rslts=c(k,dlta,aic,npar)
   for (j in 1:2)
    rslts=c(rslts,c(vv[[i]][[j]]$iterations,vv[[i]][[j]]$code,
                      vv[[i]][[j]]$estimate,vv[[i]][[j]]$tab[11,1]))
   rslts1=rbind(rslts1,rslts)
  }

  s=dimnames(rslts1)[[2]]
  s[1:8]=c('mod','delta','AIC1','AIC2','npar1','npar2','iter','code');
  s[9:(8+npar[1])]=names(vv[[1]][[1]]$estimate)
  s[(9+npar[1]):(11+npar[1])]=c('N1','iter','code')
  s=gsub('[Ss]ite 1','int',s)
  s=gsub('Survival: ','Phi-',s)
  s=gsub('Detection: ','P-',s)
  s=gsub('Superpop. size:','N-',s)
  s=gsub('Prop. Cattle ','propCat-',s)
  s=gsub('Mean ','',s)
  s=gsub('SD ','SD',s)
  s[length(s)]='N2'
  dimnames(rslts1)[[2]]=s

  rslts2=rslts1[rslts1[,4]>0,]
  #
  print(round(rslts2,3))
  x=NULL; for (i in 1:dim(rslts2)[2]) x=rbind(x,summary(rslts2[,i]))

  #dimnames(x)[[1]]=dimnames(rslts2)[[2]]
  #print(round(x,3))
  #cat(dim(rslts)[1]-dim(rslts1)[1],'/',dim(rslts)[1],'sims rejected\n')
  cat('sims ran in',round(t2[3]/60,1),'min\n')
  ##
  a=paste('phi',paste(round(c(phi_int,phi_cattle,phi_stage,phi_robel),1),collapse='_'),
            'p',paste(round(c(p_int,p_cattle,p_stage,p_robel),1),collapse='_'),sep='')
  pdf(paste(a,'.pdf',sep=''))
  par(mfrow=c(3,3),oma=c(0,0,2,0))
  ii=c(grep('P',s),grep('N[12]',s));
  for (i in ii) hist(rslts2[,i],main=' ',xlab=s[i])

  title(gsub('p','   p',gsub('_',' ',a)),outer=T)
  dev.off()
  sink(paste(a,'.out',sep=''))
  print(summary(rslts2))
  sink()
}
